package com.tzld.piaoquan.longarticle.service.local.impl;

import com.tzld.piaoquan.longarticle.common.enums.ContentStatusEnum;
import com.tzld.piaoquan.longarticle.component.RedisLock;
import com.tzld.piaoquan.longarticle.dao.mapper.aigc.ProducePlanExeRecordMapper;
import com.tzld.piaoquan.longarticle.dao.mapper.longarticle.LongArticlesTextMapper;
import com.tzld.piaoquan.longarticle.dao.mapper.longarticle.MatchVideoMapper;
import com.tzld.piaoquan.longarticle.dao.mapper.longarticle.PromotionSourceMapper;
import com.tzld.piaoquan.longarticle.model.bo.MatchContent;
import com.tzld.piaoquan.longarticle.model.po.aigc.ProducePlanExeRecord;
import com.tzld.piaoquan.longarticle.model.po.aigc.ProducePlanExeRecordExample;
import com.tzld.piaoquan.longarticle.model.po.longarticle.*;
import com.tzld.piaoquan.longarticle.service.local.KimiService;
import com.tzld.piaoquan.longarticle.utils.page.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;
import java.util.concurrent.*;

@Slf4j
@Service
public class MatchVideoServiceImpl {


    private static final String KIMI_LOCK_KEY = "kimi_lock_key_%s";

    private static final String CRAWLER_LOCK_KEY = "crawler_lock_key_%s";

    private static final String CRAWLER_COUNT_KEY = "crawler_count_key_%s";

    @Autowired
    KimiService kimiService;

    @Autowired
    private MatchVideoMapper matchVideoMapper;

    @Autowired
    private LongArticlesTextMapper longArticlesTextMapper;

    @Autowired
    private CrawlerVideoServiceImpl crawlerVideoService;

    @Autowired
    private RedisLock redisLock;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Autowired
    private ProducePlanExeRecordMapper producePlanExeRecordMapper;

    @Autowired
    private PromotionSourceMapper promotionSourceMapper;


    // 定义一个阻塞队列
    private static final ArrayBlockingQueue<MatchVideo> matchKimiVideoQueue = new ArrayBlockingQueue<>(100000);


    private static final int size = 2;
    // 定义一个线程池，设置消费线程的数量
    private static final ThreadPoolExecutor matchKimiVideoPoolExecutor = (ThreadPoolExecutor) Executors.newFixedThreadPool(size);


    // 定义一个阻塞队列
    private static final ArrayBlockingQueue<MatchVideo> matchCrawlerVideoQueue = new ArrayBlockingQueue<>(100000);

    // 定义一个线程池，设置消费线程的数量
    private static final ThreadPoolExecutor matchCrawlerVideoPoolExecutor = (ThreadPoolExecutor) Executors.newFixedThreadPool(size);

    @Transactional
    public void addMatchVideo(MatchContent matchContent) {
        MatchVideoExample example = new MatchVideoExample();
        example.createCriteria().andContentIdEqualTo(matchContent.getSourceId()).andGhIdEqualTo(matchContent.getGhId());
        long l = matchVideoMapper.countByExample(example);
        if (l > 0) {
            return;
        }
        String traceId = "search-" + UUID.randomUUID();
        MatchVideo matchVideo = new MatchVideo();
        matchVideo.setTraceId(traceId);
        matchVideo.setContentId(matchContent.getSourceId());
        matchVideo.setAccountName(matchContent.getAccountName());
        matchVideo.setGhId(matchContent.getGhId());
        matchVideo.setFlowPoolLevel(matchContent.getFlowPoolLevelTag());
        matchVideo.setContentStatus(0);
        matchVideo.setPublishFlag(matchContent.getPublishFlag());
        long timestamp = System.currentTimeMillis() / 1000;
        matchVideo.setContentStatusUpdateTime(Long.valueOf(timestamp).intValue());
        matchVideo.setRequestTimestamp(Long.valueOf(timestamp).intValue());
        matchVideoMapper.insertSelective(matchVideo);
        LongArticlesTextExample longArticlesTextExample = new LongArticlesTextExample();
        longArticlesTextExample.createCriteria().andContentIdEqualTo(matchContent.getSourceId());
        long l1 = longArticlesTextMapper.countByExample(longArticlesTextExample);
        if (l1 == 0) {
            LongArticlesText longArticlesText = new LongArticlesText();
            longArticlesText.setArticleTitle(matchContent.getTitle());
            longArticlesText.setArticleText(matchContent.getContent());
            longArticlesText.setContentId(matchContent.getSourceId());
            longArticlesText.setKimiStatus(0);
            longArticlesTextMapper.insertSelective(longArticlesText);
        }
    }

    public void matchKimiVideo() throws InterruptedException {
        if (matchKimiVideoPoolExecutor.getCorePoolSize() - matchKimiVideoPoolExecutor.getActiveCount() > 0) {
            int threadSize = matchKimiVideoPoolExecutor.getCorePoolSize() - matchKimiVideoPoolExecutor.getActiveCount();
            log.info("threadNum={}", threadSize);
            CountDownLatch countDownLatch = new CountDownLatch(threadSize);
            // 启动消费者线程
            for (int i = 0; i < threadSize; i++) {
                matchKimiVideoPoolExecutor.execute(new Thread(() -> {
                    log.info("启动Kimi搜索线程");
                    while (true) {
                        try {
                            // 超过 5 分钟没有数据，销毁当前线程
                            MatchVideo matchVideo = matchKimiVideoQueue.poll(5, TimeUnit.MINUTES); // 等待最多 5 分钟
                            if (matchVideo == null) {
                                break; // 退出当前线程
                            }
                            processKimiMatchContent(matchVideo);
                        } catch (InterruptedException e) {
                            throw new RuntimeException(e);
                        }
                    }
                    log.info("启动匹配小程序线程结束");
                    countDownLatch.countDown();
                }));
            }
            List<Integer> targetStatus = Arrays.asList(0, 1, 2);
            List<MatchVideo> matchVideos;
            //循环增加游标位置，通过主键索引过滤已经处理的数据
            do {
                Integer id = (Integer) redisTemplate.opsForValue().get("last_match_video_id");
                if (id == null) {
                    redisTemplate.opsForValue().set("last_match_video_id", 0);
                    id = 0;
                }
                MatchVideoExample example = new MatchVideoExample();
                example.createCriteria().andIdGreaterThan(id);
                example.setOrderByClause("id asc");
                Page<Object> page = new Page<>();
                page.setCurrentPage(1);
                page.setPageSize(5000);
                example.setPage(page);
                matchVideos = matchVideoMapper.selectByExample(example);
                boolean flag = true;
                for (MatchVideo matchVideo : matchVideos) {
                    if (targetStatus.contains(matchVideo.getContentStatus())) {
                        flag = false;
                        break;
                    }
                }
                if (flag) {
                    Integer lastId = matchVideos.get(matchVideos.size() - 1).getId();
                    redisTemplate.opsForValue().set("last_match_video_id", lastId);
                } else {
                    break;
                }
            } while (CollectionUtils.isEmpty(matchVideos));

            int pageNum = 1;
            do {
                Integer id = (Integer) redisTemplate.opsForValue().get("last_match_video_id");
                if (id == null) {
                    break;
                }
                int pageSize = 5000;
                MatchVideoExample example = new MatchVideoExample();
                example.createCriteria().andIdGreaterThan(id).andContentStatusEqualTo(0);
                example.setOrderByClause("id asc");
                Page<Object> page = new Page<>();
                page.setCurrentPage(pageNum);
                page.setPageSize(pageSize);
                example.setPage(page);
                matchVideos = matchVideoMapper.selectByExample(example);
                matchKimiVideoQueue.addAll(matchVideos);
                pageNum++;
            } while (CollectionUtils.isEmpty(matchVideos));
            countDownLatch.await();
        }
    }


    public void processKimiMatchContent(MatchVideo matchVideo) {
        if (matchVideo.getContentStatus() != 0) {
            return;
        }
        //1.执行kimi任务
        LongArticlesText kimiText = kimiService.getKimiText(matchVideo.getContentId());
        if (kimiText == null) {
            return;
        }
        //执行kimi任务
        if (kimiText.getKimiStatus() == 0) {
            String lockKey = String.format(KIMI_LOCK_KEY, matchVideo.getContentId());
            String lockValue = UUID.randomUUID().toString();
            boolean lock = redisLock.tryLock(lockKey, lockValue, 300, TimeUnit.SECONDS);
            if (lock) {
                boolean res = kimiService.updateKimiContent(matchVideo.getContentId());
                if (res) {
                    updateStatus(matchVideo.getId(), 1);
                }
                redisLock.unlock(lockKey, lockValue);
            }
        } else {
            //更新状态为kimi执行完成
            updateStatus(matchVideo.getId(), 1);
        }

    }

    public void matchCrawlerVideo() throws InterruptedException {
        if (matchCrawlerVideoPoolExecutor.getCorePoolSize() - matchCrawlerVideoPoolExecutor.getActiveCount() > 0) {
            int threadSize = matchCrawlerVideoPoolExecutor.getCorePoolSize() - matchCrawlerVideoPoolExecutor.getActiveCount();
            log.info("threadNum={}", threadSize);
            CountDownLatch countDownLatch = new CountDownLatch(threadSize);
            // 启动消费者线程
            for (int i = 0; i < threadSize; i++) {
                matchCrawlerVideoPoolExecutor.execute(new Thread(() -> {
                    log.info("启动匹配小程序线程");
                    while (true) {
                        try {
                            // 超过 5 分钟没有数据，销毁当前线程
                            MatchVideo matchVideo = matchCrawlerVideoQueue.poll(5, TimeUnit.MINUTES); // 等待最多 5 分钟
                            if (matchVideo == null) {
                                break; // 退出当前线程
                            }
                            processCrawlerMatchContent(matchVideo);
                        } catch (InterruptedException e) {
                            throw new RuntimeException(e);
                        }
                    }
                    log.info("启动匹配小程序线程结束");
                    countDownLatch.countDown();
                }));
            }
            int pageNum = 1;
            List<MatchVideo> matchVideos;
            List<Integer> status = Arrays.asList(1, 2);
            do {
                Integer id = (Integer) redisTemplate.opsForValue().get("last_match_video_id");
                if (id == null) {
                    break;
                }
                int pageSize = 5000;
                MatchVideoExample example = new MatchVideoExample();
                example.createCriteria().andIdGreaterThan(id).andContentStatusIn(status);
                example.setOrderByClause("id asc");
                Page<Object> page = new Page<>();
                page.setCurrentPage(pageNum);
                page.setPageSize(pageSize);
                example.setPage(page);
                matchVideos = matchVideoMapper.selectByExample(example);
                matchKimiVideoQueue.addAll(matchVideos);
                pageNum++;
            } while (CollectionUtils.isEmpty(matchVideos));
            countDownLatch.await();
        }
    }


    public void processCrawlerMatchContent(MatchVideo matchVideo) {
        if (matchVideo.getContentStatus() != 1 || matchVideo.getContentStatus() != 2) {
            return;
        }
        boolean existCrawlerVideo = crawlerVideoService.existCrawlerVideo(matchVideo.getContentId());
        if (!existCrawlerVideo) {
            MatchVideoExample example = new MatchVideoExample();
            example.createCriteria().andContentIdEqualTo(matchVideo.getContentId());
            List<MatchVideo> matchVideos = matchVideoMapper.selectByExample(example);
            if (!CollectionUtils.isEmpty(matchVideos)) {
                for (MatchVideo matchVideo1 : matchVideos) {
                    if (matchVideo1.getId().equals(matchVideo.getId())) {
                        continue;
                    }
                    if (ContentStatusEnum.isFail(matchVideo1.getContentStatus())) {
                        updateStatus(matchVideo.getId(), matchVideo1.getContentStatus());
                        return;
                    }
                }
            }
            String lockKey = String.format(CRAWLER_LOCK_KEY, matchVideo.getContentId());
            String lockValue = UUID.randomUUID().toString();
            boolean lock = redisLock.tryLock(lockKey, lockValue, 600, TimeUnit.SECONDS);
            if (lock) {
                LongArticlesText kimiText = kimiService.getKimiText(matchVideo.getContentId());
                String rootContentId = getRootContentId(matchVideo.getContentId());
                boolean res = crawlerVideoService.addCrawlerVideo(matchVideo.getContentId(), rootContentId, kimiText);
                if (res) {
                    updateStatus(matchVideo.getId(), 3);
                } else {
                    //匹配失败记录
                    String countKey = String.format(CRAWLER_COUNT_KEY, matchVideo.getContentId());
                    Integer count = (Integer) redisTemplate.opsForValue().get(countKey);
                    if (count != null && count >= 3) {
                        //更新状态为失败
                        updateStatus(matchVideo.getId(), ContentStatusEnum.ERROR_99.getStatusCode());
                    } else {
                        if (count == null) {
                            redisTemplate.opsForValue().set(countKey, 1, 3, TimeUnit.DAYS);
                        } else {
                            redisTemplate.opsForValue().set(countKey, count + 1, 3, TimeUnit.DAYS);
                        }
                    }
                }
                redisLock.unlock(lockKey, lockValue);
            }
        } else {
            //更新状态为etl执行完成
            updateStatus(matchVideo.getId(), ContentStatusEnum.SUCCESS_3.getStatusCode());
        }
    }

    private String getRootContentId(String contentId) {
//        ProducePlanExeRecordExample example = new ProducePlanExeRecordExample();
//        example.createCriteria().andPlanExeIdEqualTo(contentId);
//        List<ProducePlanExeRecord> producePlanExeRecords = producePlanExeRecordMapper.selectByExample(example);
//        if (!CollectionUtils.isEmpty(producePlanExeRecords)) {
//            String channelContentId = producePlanExeRecords.get(0).getChannelContentId();
//            if (channelContentId == null) {
//                return null;
//            }
//            PromotionSourceExample promotionSourceExample = new PromotionSourceExample();
//            promotionSourceExample.createCriteria().andChannelContentIdEqualTo(channelContentId);
//            List<PromotionSource> promotionSources = promotionSourceMapper.selectByExample(promotionSourceExample);
//            if (CollectionUtils.isEmpty(promotionSources)) {
//                return null;
//            }
//            return promotionSources.get(0).getRootProduceContentId();
//        }
        return null;
    }

    private void updateStatus(Integer id, Integer status) {
        MatchVideo matchVideo = new MatchVideo();
        matchVideo.setId(id);
        matchVideo.setContentStatus(status);
        matchVideoMapper.updateByPrimaryKeySelective(matchVideo);
    }


}



